/*
 * hello-glib-threads.c --
 *
 *      This program uses the glib library to print a short message
 *      using a worker thread and an asynchronous queue.
 */

#include <stdlib.h>
#include <glib.h>

static gpointer worker(gpointer data)
{
    GAsyncQueue *pipe = (GAsyncQueue *) data;
    g_assert(pipe);
    g_async_queue_ref(pipe);                    // ignore race condition
    while (1) {
        char *msg = g_async_queue_try_pop(pipe);
        if (! msg) break;
        g_print("%s", msg);
    }
    g_async_queue_unref(pipe);
    g_print("\n");
    return NULL;
}

int main(void)
{
    gchar *msgs[] = { "hello", " ", "world", NULL };
    GAsyncQueue *pipe;
    GThread *handle;

    pipe = g_async_queue_new();
    g_return_val_if_fail(pipe != NULL, EXIT_FAILURE);
    handle = g_thread_new(NULL, worker, pipe);
    g_return_val_if_fail(handle != NULL, EXIT_FAILURE);

    for (int i = 0; msgs[i]; i++) {
        g_async_queue_push(pipe, msgs[i]);
    }
    (void) g_thread_join(handle);
    g_async_queue_unref(pipe);
    return EXIT_SUCCESS;
}
